<?php

namespace Drupal\eu_cookie_compliance\Controller;

use Drupal\Component\Serialization\Json;
use Drupal\Core\Ajax\AjaxResponse;
use Drupal\Core\Ajax\CloseDialogCommand;
use Drupal\Core\Ajax\OpenModalDialogCommand;
use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Extension\ModuleHandlerInterface;
use Drupal\Core\Form\FormBuilderInterface;
use Drupal\Core\Form\FormState;
use Drupal\Core\Form\FormStateInterface;
use Drupal\eu_cookie_compliance\Ajax\UpdateCookiePrefsListCommand;
use Drupal\eu_cookie_compliance\Entity\CookieBanner;
use Drupal\eu_cookie_compliance\Form\CookieBannerForm;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\RequestStack;
use Symfony\Component\HttpKernel\HttpKernelInterface;

/**
 * Returns responses for Cookie Compliance routes.
 */
class EuCookieComplianceController extends ControllerBase {

  /**
   * Module Handler.
   *
   * @var \Drupal\Core\Extension\ModuleHandlerInterface
   */
  protected $moduleHandler;

  /**
   * Form Builder.
   *
   * @var \Drupal\Core\Form\FormBuilderInterface
   */
  protected $formBuilder;

  /**
   * Entity Type Manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * Http Kernel.
   *
   * @var \Symfony\Component\HttpKernel\HttpKernelInterface
   */
  protected $httpKernel;

  /**
   * Request stack.
   *
   * @var \Symfony\Component\HttpFoundation\RequestStack
   */
  protected $requestStack;

  /**
   * Creates a CookieComplianceSetForm instance.
   *
   * @param \Drupal\Core\Extension\ModuleHandlerInterface $module_handler
   *   Entity Type manager.
   * @param \Drupal\Core\Form\FormBuilderInterface $form_builder
   *   Form builder.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   Entity Type Manager.
   * @param \Symfony\Component\HttpKernel\HttpKernelInterface $http_kernel
   *   Http Kernel.
   * @param \Symfony\Component\HttpFoundation\RequestStack $request_stack
   *   Request stack.
   */
  public function __construct(ModuleHandlerInterface $module_handler, FormBuilderInterface $form_builder, EntityTypeManagerInterface $entity_type_manager, HttpKernelInterface $http_kernel, RequestStack $request_stack) {
    $this->moduleHandler = $module_handler;
    $this->formBuilder = $form_builder;
    $this->entityTypeManager = $entity_type_manager;
    $this->httpKernel = $http_kernel;
    $this->requestStack = $request_stack;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('module_handler'),
      $container->get('form_builder'),
      $container->get('entity_type.manager'),
      $container->get('http_kernel'),
      $container->get('request_stack')
    );
  }

  /**
   * Builds the response.
   */
  public function buildCookieBannerDialog() {
    $form_state = new FormState();
    $form = new CookieBannerForm($this->entityTypeManager, $this->moduleHandler, $this->requestStack);
    $form->setEntity(new CookieBanner([], 'cookie_banner'));
    $entity = $form->getEntity();
    $entity->enforceIsNew();
    $form->setEntity($entity);
    $form->setModuleHandler($this->moduleHandler);
    $form_state->setFormObject($form);
    $form = $this->formBuilder->buildForm($form, $form_state);

    // Send the form errors back to the form on error.
    $form_errors = $form_state->getErrors();
    if ($form_errors) {
      $form['status_messages'] = [
        '#type' => 'status_messages',
        'message_list' => $form_errors,
        '#weight' => -10,
      ];
      $response = new AjaxResponse();
      $response->addCommand(new CloseDialogCommand('#drupal-modal'));
      $response->addCommand(new OpenModalDialogCommand('Add Cookie Banner/Dialog', $form, ['width' => 640]));
      $response->setContent(Json::encode($response->getCommands()));
      $response->headers->add(['X-Drupal-Ajax-Token' => 1]);
      $response->send();
      $request = Request::createFromGlobals();
      $this->httpKernel->terminate($request, $response);
    }
    return $form;
  }

  /**
   * Builds the response.
   */
  public function buildCookieBannerEditDialog($id) {
    $form_state = new FormState();
    $form = new CookieBannerForm($this->entityTypeManager, $this->moduleHandler, $this->requestStack);
    $entity = CookieBanner::load($id);
    $form->setEntity($entity, 'cookie_banner');
    $form->setModuleHandler($this->moduleHandler);
    $form_state->setFormObject($form);
    $form = $this->formBuilder->buildForm($form, $form_state);

    // Send the form errors back to the form on error.
    $form_errors = $form_state->getErrors();
    if ($form_errors) {
      $form['status_messages'] = [
        '#type' => 'status_messages',
        'message_list' => $form_errors,
        '#weight' => -10,
      ];
      $response = new AjaxResponse();
      $response->addCommand(new CloseDialogCommand('#drupal-modal'));
      $response->addCommand(new OpenModalDialogCommand('Edit Cookie Banner/Dialog', $form, ['width' => 640]));
      $response->setContent(Json::encode($response->getCommands()));
      $response->headers->add(['X-Drupal-Ajax-Token' => 1]);
      $response->send();
      $request = Request::createFromGlobals();
      $this->httpKernel->terminate($request, $response);
    }
    return $form;
  }

  /**
   * Submit form.
   *
   * @param array $form
   *   Form.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   Form state.
   */
  public static function saveCookieBannerAndReturn(array &$form, FormStateInterface $form_state) {
    $response = new AjaxResponse();
    $response->addCommand(new CloseDialogCommand('#drupal-modal'));
    $banners = self::getList(\Drupal::service('entity_type.manager')->getStorage('cookie_banner')->loadMultiple());
    $response->addCommand(new UpdateCookiePrefsListCommand('#edit-cookie-banners', $banners, 'cookie-banners'));
    $response->setContent(Json::encode($response->getCommands()));
    $response->headers->add(['X-Drupal-Ajax-Token' => 1]);
    $response->send();
    $request = Request::createFromGlobals();
    \Drupal::service('http_kernel')->terminate($request, $response);
  }

  /**
   * Get all regions.
   *
   * @return array
   *   The regions.
   */
  protected static function getList(array $entities) : array {
    $set = [];
    foreach ($entities as $entity) {
      $set[$entity->id()] = $entity->label();
    }
    return $set;
  }

}
