<?php

namespace Drupal\eu_cookie_compliance\Entity;

use Drupal\Core\Config\Entity\ConfigEntityBase;
use Drupal\eu_cookie_compliance\CookieComplianceSetInterface;

/**
 * Defines the cookie compliance set entity type.
 *
 * @ConfigEntityType(
 *   id = "cookie_compliance_set",
 *   label = @Translation("Cookie Compliance Set"),
 *   label_collection = @Translation("Cookie Compliance Sets"),
 *   label_singular = @Translation("cookie compliance set"),
 *   label_plural = @Translation("cookie compliance sets"),
 *   label_count = @PluralTranslation(
 *     singular = "@count cookie compliance set",
 *     plural = "@count cookie compliance sets",
 *   ),
 *   handlers = {
 *     "list_builder" = "Drupal\eu_cookie_compliance\CookieComplianceSetListBuilder",
 *     "form" = {
 *       "add" = "Drupal\eu_cookie_compliance\Form\CookieComplianceSetForm",
 *       "edit" = "Drupal\eu_cookie_compliance\Form\CookieComplianceSetForm",
 *       "delete" = "Drupal\Core\Entity\EntityDeleteForm"
 *     }
 *   },
 *   config_prefix = "cookie_compliance_set",
 *   admin_permission = "administer eu cookie compliance banner",
 *   links = {
 *     "collection" = "/admin/config/system/eu-cookie-compliance/cookie-compliance-set",
 *     "add-form" = "/admin/config/system/eu-cookie-compliance/cookie-compliance-set/add",
 *     "edit-form" = "/admin/config/system/eu-cookie-compliance/cookie-compliance-set/{cookie_compliance_set}",
 *     "delete-form" = "/admin/config/system/eu-cookie-compliance/cookie-compliance-set/{cookie_compliance_set}/delete"
 *   },
 *   entity_keys = {
 *     "id" = "id",
 *     "label" = "label",
 *     "uuid" = "uuid"
 *   },
 *   config_export = {
 *     "id",
 *     "label",
 *     "enabled",
 *     "consent_storage_method",
 *     "cookie_banners",
 *     "regions",
 *     "domains",
 *     "custom_regions",
 *     "languages",
 *     "method",
 *     "privacy_policy_link",
 *     "open_privacy_link_in_new_window",
 *     "privacy_policy_version",
 *     "cookie_name",
 *     "cookie_same_site",
 *     "cookie_value_disagreed",
 *     "cookie_value_agreed",
 *     "cookie_lifetime",
 *     "add_remove_banner_on_domain",
 *     "add_remove_domains",
 *     "exclude_paths",
 *     "exclude_admin_theme",
 *     "exclude_uid_1",
 *     "prompt_at_every_session",
 *     "domain",
 *     "set_cookie_for_all_domains",
 *     "reload_page",
 *     "reload_options",
 *     "reload_routes_list",
 *     "set_cookie_session_zero_on_disagree",
 *     "consent_by_clicking",
 *     "enable_policy_versioning",
 *     "disabled_javascripts",
 *     "automatic_cookies_removal",
 *     "allowed_cookies"
 *   }
 * )
 */
class CookieComplianceSet extends ConfigEntityBase implements CookieComplianceSetInterface {

  /**
   * The cookie compliance set ID.
   *
   * @var string
   */
  protected $id;

  /**
   * The cookie compliance set label.
   *
   * @var string
   */
  protected $label;

  /**
   * Enabled cookie banners.
   *
   * @var array
   */
  protected $cookie_banners;

  /**
   * Enabled regions.
   *
   * @var array
   */
  protected $regions;

  /**
   * Custom regions.
   *
   * @var array
   */
  protected $custom_regions;

  /**
   * Enabled languages.
   *
   * @var array
   */
  protected $languages;

  /**
   * Enabled domains.
   *
   * @var array
   */
  protected $domains;

  /**
   * Add/remove banner on specified domains.
   *
   * @var string
   */
  protected $add_remove_banner_on_domain;

  /**
   * Domains list.
   *
   * @var string
   */
  protected $add_remove_domains;

  /**
   * Consent storage method.
   *
   * @var string
   */
  protected $consent_storage_method;

  /**
   * Cookie lifetime.
   *
   * @var int
   */
  protected $cookie_lifetime;

  /**
   * Cookie name.
   *
   * @var string
   */
  protected $cookie_name;

  /**
   * Same Site.
   *
   * @var string
   */
  protected $cookie_same_site;

  /**
   * Cookie value (disagreed).
   *
   * @var string
   */
  protected $cookie_value_agreed;

  /**
   * Cookie value (agreed).
   *
   * @var string
   */
  protected $cookie_value_disagreed;

  /**
   * Domain.
   *
   * @var string
   */
  protected $domain;

  /**
   * Enabled.
   *
   * @var bool
   */
  protected $enabled;

  /**
   * Exclude admin pages.
   *
   * @var bool
   */
  protected $exclude_admin_theme;

  /**
   * Exclude paths.
   *
   * @var string
   */
  protected $exclude_paths;

  /**
   * Don't show the banner for site administrators (including UID 1).
   *
   * @var bool
   */
  protected $exclude_uid_1;

  /**
   * Consent method.
   *
   * @var string
   */
  protected $method;

  /**
   * Open privacy policy link in a new window.
   *
   * @var bool
   */
  protected $open_privacy_link_in_new_window;

  /**
   * Privacy policy link.
   *
   * @var string
   */
  protected $privacy_policy_link;

  /**
   * Privacy policy version.
   *
   * @var string
   */
  protected $privacy_policy_version;

  /**
   * Prompt for consent (from the same user) at every new browser session.
   *
   * @var bool
   */
  protected $prompt_at_every_session;

  /**
   * Include/exclude specified routes for reloading.
   *
   * @var string
   */
  protected $reload_options;

  /**
   * Reload page after user clicks the 'Agree' button.
   *
   * @var bool
   */
  protected $reload_page;

  /**
   * Routes list.
   *
   * @var string
   */
  protected $reload_routes_list;

  /**
   * Allow the cookie to be set for all sites on the same domain.
   *
   * @var bool
   */
  protected $set_cookie_for_all_domains;

  /**
   * Set cookie lifetime to 0 if user clicks 'Decline' or 'Withdraw consent'.
   *
   * @var bool
   */
  protected $set_cookie_session_zero_on_disagree;

  /**
   * Whether to allow consent by clicking.
   *
   * @var bool
   */
  protected $consent_by_clicking;

  /**
   * Whether to allow policy versioning.
   *
   * @var bool
   */
  protected $enable_policy_versioning;

  /**
   * Disabled Javascripts.
   *
   * @var string
   */
  protected $disabled_javascripts;

  /**
   * Whether to remove cookies automatically.
   *
   * @var bool
   */
  protected $automatic_cookies_removal;

  /**
   * Allowed cookies.
   *
   * @var string
   */
  protected $allowed_cookies;

}
