<?php

namespace Drupal\eu_cookie_compliance\Form;

use Drupal\Core\Entity\EntityForm;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Extension\ModuleHandlerInterface;
use Drupal\Core\Form\FormStateInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\RequestStack;

/**
 * Cookie Banner form.
 *
 * @property \Drupal\eu_cookie_compliance\CookieBannerInterface $entity
 */
class CookieBannerForm extends EntityForm {

  /**
   * The filter format storage.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * The module handler.
   *
   * @var \Drupal\Core\Extension\ModuleHandlerInterface
   */
  protected $moduleHandler;

  /**
   * The request stack.
   *
   * @var \Symfony\Component\HttpFoundation\RequestStack
   */
  protected $requestStack;

  /**
   * Constructs an EuCookieComplianceConfigForm object.
   *
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager.
   * @param \Drupal\Core\Extension\ModuleHandlerInterface $module_handler
   *   The module handler.
   * @param \Symfony\Component\HttpFoundation\RequestStack $request_stack
   *   The request stack.
   */
  public function __construct(EntityTypeManagerInterface $entity_type_manager, ModuleHandlerInterface $module_handler, RequestStack $request_stack) {
    $this->entityTypeManager = $entity_type_manager;
    $this->moduleHandler = $module_handler;
    $this->requestStack = $request_stack;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('entity_type.manager'),
      $container->get('module_handler'),
      $container->get('request_stack')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function form(array $form, FormStateInterface $form_state) {

    // @todo Provide a wizard for selecting the banner style and provide defaults.
    // @todo Global setting on whether to style the banner in settings or theme css.
    $form = parent::form($form, $form_state);

    if ($this->moduleHandler->moduleExists('coloris')) {
      $color_widget = 'coloriswidget';
      $form['#attached']['library'][] = 'coloris/element.coloris';
    }
    else {
      $color_widget = 'textfield';
    }

    $id = $this->entity->id();

    $form['is_new'] = [
      '#type' => 'hidden',
      '#value' => $id === NULL ? 'true' : 'false',
    ];

    $form['label'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Label'),
      '#maxlength' => 255,
      '#default_value' => $this->entity->label(),
      '#required' => TRUE,
    ];

    $form['id'] = [
      '#type' => 'machine_name',
      '#default_value' => $this->entity->id(),
      '#machine_name' => [
        'exists' => '\Drupal\eu_cookie_compliance\Entity\CookieBanner::load',
      ],
      '#disabled' => !$this->entity->isNew(),
    ];

    $form['type'] = [
      '#type' => 'select',
      '#title' => $this->t('Banner/dialog type'),
      '#options' => [
        'banner_at_bottom' => $this->t('Banner at the bottom'),
        'banner_at_top' => $this->t('Banner at the top'),
        'banner_at_top_fixed' => $this->t('Banner at the top, fixed'),
        'dialog' => $this->t('Dialog'),
        'modal_dialog' => $this->t('Modal dialog, covering page with an overlay'),
        'side_banner_left' => $this->t('Side banner, left side'),
        'side_banner_right' => $this->t('Side banner, right side'),
      ],
      '#default_value' => $this->entity->get('type'),
    ];
    $form['style'] = [
      '#type' => 'select',
      '#title' => $this->t('Banner style'),
      '#options' => [
        'modern' => $this->t('Modern'),
        'legacy' => $this->t('Legacy (version 1.x) banner'),
        'olivero' => $this->t('Olivero'),
      ],
      '#default_value' => $this->entity->get('style'),
    ];

    $form['formatting_group'] = [
      '#type' => 'details',
      '#title' => $this->t('Formatting'),
      '#open' => TRUE,
    ];

    $form['formatting_group']['formatting'] = [
      '#type' => 'radios',
      '#options' => [
        'css' => $this->t('I want to style the banner in my theme css.'),
        'module' => $this->t('I want to configure background, text color and border values in this window.'),
      ],
      '#default_value' => $this->entity->get('formatting') ?: 'module',
    ];

    $message_setting = $this->entity->get('message');
    if ($message_setting !== '' && $message_setting !== NULL) {
      $message_value = $message_setting['value'];
      $message_format = $message_setting['format'];
    }
    else {
      $message_value = '';
      $message_format = filter_default_format();
      $default_filter_format = filter_default_format();
      $full_html_format = $this->entityTypeManager->getStorage('filter_format')->load('full_html');
      if (($default_filter_format === 'restricted_html' || $default_filter_format === 'plain_text') && $full_html_format !== NULL && $full_html_format->get('status')) {
        $message_format = 'full_html';
      }
    }
    $form['message'] = [
      '#type' => 'text_format',
      '#title' => $this->t('Message'),
      '#default_value' => $message_value,
      '#format' => $message_format,
    ];

    $form['banner_format_group'] = [
      '#type' => 'details',
      '#title' => $this->t('Banner format'),
      '#open' => TRUE,
      '#states' => [
        'visible' => [
          "input[name='formatting']" => ['value' => 'module'],
        ],
      ],
    ];
    $form['banner_format_group']['height'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Banner/dialog height'),
      '#size' => 15,
      '#maxlength' => 12,
      '#default_value' => $this->entity->get('height'),
      '#description' => $this->t('Leave empty for automatically adjusted height.'),
      '#states' => [
        'visible' => [
          "input[name='formatting']" => ['value' => 'module'],
        ],
      ],
    ];
    $form['banner_format_group']['width'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Banner width in units (px etc) or a percentage value'),
      '#default_value' => $this->entity->get('width'),
      '#field_suffix' => $this->t('in units or %'),
      '#size' => 15,
      '#maxlength' => 12,
      '#states' => [
        'visible' => [
          "input[name='formatting']" => ['value' => 'module'],
        ],
      ],
    ];
    $form['banner_format_group']['banner_dialog_text_color'] = [
      '#type' => $color_widget,
      '#alpha' => FALSE,
      '#data_theme' => 'polaroid',
      '#size' => 8,
      '#maxlength' => 7,
      '#title' => $this->t('Banner/dialog text color'),
      '#default_value' => $this->entity->get('banner_dialog_text_color'),
      '#states' => [
        'visible' => [
          "input[name='formatting']" => ['value' => 'module'],
        ],
      ],
      '#element_validate' => [[$this, 'validateHex']],
    ];
    $form['banner_format_group']['banner_dialog_background_color'] = [
      '#type' => $color_widget,
      '#alpha' => FALSE,
      '#data_theme' => 'polaroid',
      '#size' => 8,
      '#maxlength' => 7,
      '#title' => $this->t('Banner/dialog background color'),
      '#default_value' => $this->entity->get('banner_dialog_background_color'),
      '#states' => [
        'visible' => [
          "input[name='formatting']" => ['value' => 'module'],
        ],
      ],
      '#element_validate' => [[$this, 'validateHex']],
    ];
    $form['banner_format_group']['banner_dialog_border_width'] = [
      '#type' => 'textfield',
      '#size' => 15,
      '#maxlength' => 12,
      '#title' => $this->t('Banner/dialog border width'),
      '#default_value' => $this->entity->get('banner_dialog_border_width'),
      '#states' => [
        'visible' => [
          "input[name='formatting']" => ['value' => 'module'],
        ],
      ],
    ];
    $form['banner_format_group']['banner_dialog_border_color'] = [
      '#type' => $color_widget,
      '#alpha' => FALSE,
      '#data_theme' => 'polaroid',
      '#size' => 8,
      '#maxlength' => 7,
      '#title' => $this->t('Banner/dialog border color'),
      '#default_value' => $this->entity->get('banner_dialog_border_color'),
      '#states' => [
        'visible' => [
          "input[name='formatting']" => ['value' => 'module'],
        ],
      ],
      '#element_validate' => [[$this, 'validateHex']],
    ];
    $form['buttons'] = [
      '#type' => 'details',
      '#title' => $this->t('Buttons'),
      '#open' => TRUE,
    ];
    $form['buttons']['enable_privacy_policy_button'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Enable "Privacy policy" button'),
      '#description' => $this->t('The "Privacy policy" button opens the privacy policy link defined in the Cookie Compliance Set. If you would like to display a link to the privacy policy within the banner text, please enter it as part of the banner message.'),
      '#default_value' => $this->entity->get('enable_privacy_policy_button'),
    ];
    $form['buttons']['privacy_policy_button_label'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Privacy policy button label'),
      '#default_value' => $this->entity->get('privacy_policy_button_label'),
      '#states' => [
        'visible' => [
          "input[name='enable_privacy_policy_button']" => ['checked' => TRUE],
        ],
      ],
    ];
    $form['buttons']['privacy_policy_button_text_color'] = [
      '#type' => $color_widget,
      '#alpha' => FALSE,
      '#data_theme' => 'polaroid',
      '#size' => 8,
      '#maxlength' => 7,
      '#title' => $this->t('Privacy policy button text color'),
      '#default_value' => $this->entity->get('privacy_policy_button_text_color'),
      '#states' => [
        'visible' => [
          "input[name='enable_privacy_policy_button']" => ['checked' => TRUE],
          "input[name='formatting']" => ['value' => 'module'],
        ],
      ],
      '#element_validate' => [[$this, 'validateHex']],
    ];
    $form['buttons']['privacy_policy_button_background_color'] = [
      '#type' => $color_widget,
      '#alpha' => FALSE,
      '#data_theme' => 'polaroid',
      '#size' => 8,
      '#maxlength' => 7,
      '#title' => $this->t('Privacy policy button background color'),
      '#default_value' => $this->entity->get('privacy_policy_button_background_color'),
      '#states' => [
        'visible' => [
          "input[name='enable_privacy_policy_button']" => ['checked' => TRUE],
          "input[name='formatting']" => ['value' => 'module'],
        ],
      ],
      '#element_validate' => [[$this, 'validateHex']],
    ];
    $form['buttons']['privacy_policy_button_border_width'] = [
      '#type' => 'textfield',
      '#size' => 15,
      '#maxlength' => 12,
      '#title' => $this->t('Privacy policy button border width'),
      '#default_value' => $this->entity->get('privacy_policy_button_border_width'),
      '#states' => [
        'visible' => [
          "input[name='enable_privacy_policy_button']" => ['checked' => TRUE],
          "input[name='formatting']" => ['value' => 'module'],
        ],
      ],
    ];
    $form['buttons']['privacy_policy_button_border_color'] = [
      '#type' => $color_widget,
      '#alpha' => FALSE,
      '#data_theme' => 'polaroid',
      '#size' => 8,
      '#maxlength' => 7,
      '#title' => $this->t('Privacy policy button border color'),
      '#default_value' => $this->entity->get('privacy_policy_button_border_color'),
      '#states' => [
        'visible' => [
          "input[name='enable_privacy_policy_button']" => ['checked' => TRUE],
          "input[name='formatting']" => ['value' => 'module'],
        ],
      ],
      '#element_validate' => [[$this, 'validateHex']],
    ];
    $form['buttons']['enable_accept_all_button'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Enable "Accept all" button'),
      '#description' => $this->t('The "Accept all" button accepts all categories when using a banner with categories. Otherwise, it accepts the cookie policy and closes the banner.'),
      '#default_value' => $this->entity->get('enable_accept_all_button'),
    ];
    $form['buttons']['accept_all_button_label'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Accept all button label'),
      '#default_value' => $this->entity->get('accept_all_button_label'),
      '#states' => [
        'visible' => [
          "input[name='enable_accept_all_button']" => ['checked' => TRUE],
        ],
      ],
    ];
    $form['buttons']['accept_all_button_text_color'] = [
      '#type' => $color_widget,
      '#alpha' => FALSE,
      '#data_theme' => 'polaroid',
      '#size' => 8,
      '#maxlength' => 7,
      '#title' => $this->t('Accept all button text color'),
      '#default_value' => $this->entity->get('accept_all_button_text_color'),
      '#states' => [
        'visible' => [
          "input[name='enable_accept_all_button']" => ['checked' => TRUE],
          "input[name='formatting']" => ['value' => 'module'],
        ],
      ],
      '#element_validate' => [[$this, 'validateHex']],
    ];
    $form['buttons']['accept_all_button_background_color'] = [
      '#type' => $color_widget,
      '#alpha' => FALSE,
      '#data_theme' => 'polaroid',
      '#size' => 8,
      '#maxlength' => 7,
      '#title' => $this->t('Accept all button background color'),
      '#default_value' => $this->entity->get('accept_all_button_background_color'),
      '#states' => [
        'visible' => [
          "input[name='enable_accept_all_button']" => ['checked' => TRUE],
          "input[name='formatting']" => ['value' => 'module'],
        ],
      ],
      '#element_validate' => [[$this, 'validateHex']],
    ];
    $form['buttons']['accept_all_button_border_width'] = [
      '#type' => 'textfield',
      '#size' => 15,
      '#maxlength' => 12,
      '#title' => $this->t('Accept all button border width'),
      '#default_value' => $this->entity->get('accept_all_button_border_width'),
      '#states' => [
        'visible' => [
          "input[name='enable_accept_all_button']" => ['checked' => TRUE],
          "input[name='formatting']" => ['value' => 'module'],
        ],
      ],
    ];
    $form['buttons']['accept_all_button_border_color'] = [
      '#type' => $color_widget,
      '#alpha' => FALSE,
      '#data_theme' => 'polaroid',
      '#size' => 8,
      '#maxlength' => 7,
      '#title' => $this->t('Accept all button border color'),
      '#default_value' => $this->entity->get('accept_all_button_border_color'),
      '#states' => [
        'visible' => [
          "input[name='enable_accept_all_button']" => ['checked' => TRUE],
          "input[name='formatting']" => ['value' => 'module'],
        ],
      ],
      '#element_validate' => [[$this, 'validateHex']],
    ];
    $form['buttons']['enable_reject_all_button'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Enable "Reject all" button'),
      '#description' => $this->t('The "Reject all" button accepts only categories that are set to be checked and disabled when categories are being used. Otherwise, it rejects the cookie policy.'),
      '#default_value' => $this->entity->get('enable_reject_all_button'),
    ];
    $form['buttons']['reject_all_button_label'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Reject all button label'),
      '#default_value' => $this->entity->get('reject_all_button_label'),
      '#states' => [
        'visible' => [
          "input[name='enable_reject_all_button']" => ['checked' => TRUE],
        ],
      ],
    ];
    $form['buttons']['reject_all_button_text_color'] = [
      '#type' => $color_widget,
      '#alpha' => FALSE,
      '#data_theme' => 'polaroid',
      '#size' => 8,
      '#maxlength' => 7,
      '#title' => $this->t('Reject all button text color'),
      '#default_value' => $this->entity->get('reject_all_button_text_color'),
      '#states' => [
        'visible' => [
          "input[name='enable_reject_all_button']" => ['checked' => TRUE],
          "input[name='formatting']" => ['value' => 'module'],
        ],
      ],
      '#element_validate' => [[$this, 'validateHex']],
    ];
    $form['buttons']['reject_all_button_background_color'] = [
      '#type' => $color_widget,
      '#alpha' => FALSE,
      '#data_theme' => 'polaroid',
      '#size' => 8,
      '#maxlength' => 7,
      '#title' => $this->t('Reject all button background color'),
      '#default_value' => $this->entity->get('reject_all_button_background_color'),
      '#states' => [
        'visible' => [
          "input[name='enable_reject_all_button']" => ['checked' => TRUE],
          "input[name='formatting']" => ['value' => 'module'],
        ],
      ],
      '#element_validate' => [[$this, 'validateHex']],
    ];
    $form['buttons']['reject_all_button_border_width'] = [
      '#type' => 'textfield',
      '#size' => 15,
      '#maxlength' => 12,
      '#title' => $this->t('Reject all button border width'),
      '#default_value' => $this->entity->get('reject_all_button_border_width'),
      '#states' => [
        'visible' => [
          "input[name='enable_reject_all_button']" => ['checked' => TRUE],
          "input[name='formatting']" => ['value' => 'module'],
        ],
      ],
    ];
    $form['buttons']['reject_all_button_border_color'] = [
      '#type' => $color_widget,
      '#alpha' => FALSE,
      '#data_theme' => 'polaroid',
      '#size' => 8,
      '#maxlength' => 7,
      '#title' => $this->t('Reject all button border color'),
      '#default_value' => $this->entity->get('reject_all_button_border_color'),
      '#states' => [
        'visible' => [
          "input[name='enable_reject_all_button']" => ['checked' => TRUE],
          "input[name='formatting']" => ['value' => 'module'],
        ],
      ],
      '#element_validate' => [[$this, 'validateHex']],
    ];
    $form['buttons']['enable_preferences_button'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Enable "Preferences" button'),
      '#description' => $this->t('The "Preferences" button opens a second banner/dialog, normally that banner/dialog is used for display and selection of categories.'),
      '#default_value' => $this->entity->get('enable_preferences_button'),
    ];
    $form['buttons']['preferences_button_label'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Preferences button label'),
      '#default_value' => $this->entity->get('preferences_button_label'),
      '#states' => [
        'visible' => [
          "input[name='enable_preferences_button']" => ['checked' => TRUE],
        ],
      ],
    ];
    $form['buttons']['preferences_button_text_color'] = [
      '#type' => $color_widget,
      '#alpha' => FALSE,
      '#data_theme' => 'polaroid',
      '#size' => 8,
      '#maxlength' => 7,
      '#title' => $this->t('Preferences button text color'),
      '#default_value' => $this->entity->get('preferences_button_text_color'),
      '#states' => [
        'visible' => [
          "input[name='enable_preferences_button']" => ['checked' => TRUE],
          "input[name='formatting']" => ['value' => 'module'],
        ],
      ],
      '#element_validate' => [[$this, 'validateHex']],
    ];
    $form['buttons']['preferences_button_background_color'] = [
      '#type' => $color_widget,
      '#alpha' => FALSE,
      '#data_theme' => 'polaroid',
      '#size' => 8,
      '#maxlength' => 7,
      '#title' => $this->t('Preferences button background color'),
      '#default_value' => $this->entity->get('preferences_button_background_color'),
      '#states' => [
        'visible' => [
          "input[name='enable_preferences_button']" => ['checked' => TRUE],
          "input[name='formatting']" => ['value' => 'module'],
        ],
      ],
      '#element_validate' => [[$this, 'validateHex']],
    ];
    $form['buttons']['preferences_button_border_width'] = [
      '#type' => 'textfield',
      '#size' => 15,
      '#maxlength' => 12,
      '#title' => $this->t('Preferences button border width'),
      '#default_value' => $this->entity->get('preferences_button_border_width'),
      '#states' => [
        'visible' => [
          "input[name='enable_preferences_button']" => ['checked' => TRUE],
          "input[name='formatting']" => ['value' => 'module'],
        ],
      ],
    ];
    $form['buttons']['preferences_button_border_color'] = [
      '#type' => $color_widget,
      '#alpha' => FALSE,
      '#data_theme' => 'polaroid',
      '#size' => 8,
      '#maxlength' => 7,
      '#title' => $this->t('Preferences button border color'),
      '#default_value' => $this->entity->get('preferences_button_border_color'),
      '#states' => [
        'visible' => [
          "input[name='enable_preferences_button']" => ['checked' => TRUE],
          "input[name='formatting']" => ['value' => 'module'],
        ],
      ],
      '#element_validate' => [[$this, 'validateHex']],
    ];

    $banner_list = $this->entityTypeManager->getStorage('cookie_banner')->loadMultiple();

    $banners = [];
    foreach ($banner_list as $entity) {
      $banners[$entity->id()] = $entity->label();
    }

    $form['buttons']['preferences_open_dialog_id'] = [
      '#type' => 'select',
      '#title' => $this->t('Preferences dialog to open'),
      '#options' => $banners,
      '#default_value' => $this->entity->get('preferences_open_dialog_id') ?: '',
      '#states' => [
        'visible' => [
          "input[name='enable_preferences_button']" => ['checked' => TRUE],
        ],
      ],
    ];
    $form['buttons']['enable_save_preferences_button'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Enable "Save preferences" button'),
      '#description' => $this->t('The "Save preferences" button saves the selected categories or accepts the cookie policy and closes the banner.'),
      '#default_value' => $this->entity->get('enable_save_preferences_button'),
    ];
    $form['buttons']['save_preferences_button_label'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Save preferences button label'),
      '#default_value' => $this->entity->get('save_preferences_button_label'),
      '#states' => [
        'visible' => [
          "input[name='enable_save_preferences_button']" => ['checked' => TRUE],
        ],
      ],
    ];
    $form['buttons']['save_preferences_button_text_color'] = [
      '#type' => $color_widget,
      '#alpha' => FALSE,
      '#data_theme' => 'polaroid',
      '#size' => 8,
      '#maxlength' => 7,
      '#title' => $this->t('Save preferences button text color'),
      '#default_value' => $this->entity->get('save_preferences_button_text_color'),
      '#states' => [
        'visible' => [
          "input[name='enable_save_preferences_button']" => ['checked' => TRUE],
          "input[name='formatting']" => ['value' => 'module'],
        ],
      ],
      '#element_validate' => [[$this, 'validateHex']],
    ];
    $form['buttons']['save_preferences_button_background_color'] = [
      '#type' => $color_widget,
      '#alpha' => FALSE,
      '#data_theme' => 'polaroid',
      '#size' => 8,
      '#maxlength' => 7,
      '#title' => $this->t('Save preferences button background color'),
      '#default_value' => $this->entity->get('save_preferences_button_background_color'),
      '#states' => [
        'visible' => [
          "input[name='enable_save_preferences_button']" => ['checked' => TRUE],
          "input[name='formatting']" => ['value' => 'module'],
        ],
      ],
      '#element_validate' => [[$this, 'validateHex']],
    ];
    $form['buttons']['save_preferences_button_border_width'] = [
      '#type' => 'textfield',
      '#size' => 15,
      '#maxlength' => 12,
      '#title' => $this->t('Save preferences button border width'),
      '#default_value' => $this->entity->get('save_preferences_button_border_width'),
      '#states' => [
        'visible' => [
          "input[name='enable_save_preferences_button']" => ['checked' => TRUE],
          "input[name='formatting']" => ['value' => 'module'],
        ],
      ],
    ];
    $form['buttons']['save_preferences_button_border_color'] = [
      '#type' => $color_widget,
      '#alpha' => FALSE,
      '#data_theme' => 'polaroid',
      '#size' => 8,
      '#maxlength' => 7,
      '#title' => $this->t('Save preferences button border color'),
      '#default_value' => $this->entity->get('save_preferences_button_border_color'),
      '#states' => [
        'visible' => [
          "input[name='enable_save_preferences_button']" => ['checked' => TRUE],
          "input[name='formatting']" => ['value' => 'module'],
        ],
      ],
      '#element_validate' => [[$this, 'validateHex']],
    ];
    $form['buttons']['enable_withdraw_button'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Enable "Withdraw consent" message and action button'),
      '#description' => $this->t('The "Withdraw consent" button can be seen when consent has been given. It resets all cookies from this module and displays the initial banner again.'),
      '#default_value' => $this->entity->get('enable_withdraw_button'),
    ];
    $message_setting = $this->entity->get('withdraw_message');
    if ($message_setting !== '' && $message_setting !== NULL) {
      $message_value = $message_setting['value'];
      $message_format = $message_setting['format'];
    }
    else {
      $message_value = '';
      $message_format = filter_default_format();
      $default_filter_format = filter_default_format();
      $full_html_format = $this->entityTypeManager->getStorage('filter_format')->load('full_html');
      if (($default_filter_format === 'restricted_html' || $default_filter_format === 'plain_text') && $full_html_format !== NULL && $full_html_format->get('status')) {
        $message_format = 'full_html';
      }
    }
    $form['buttons']['withdraw_message'] = [
      '#type' => 'text_format',
      '#title' => $this->t('Withdraw message'),
      '#default_value' => $message_value,
      '#format' => $message_format,
      '#states' => [
        'visible' => [
          "input[name='enable_withdraw_button']" => ['checked' => TRUE],
        ],
      ],
    ];
    $form['buttons']['withdraw_button_label'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Withdraw consent action button label'),
      '#default_value' => $this->entity->get('withdraw_button_label'),
      '#states' => [
        'visible' => [
          "input[name='enable_withdraw_button']" => ['checked' => TRUE],
        ],
      ],
    ];
    $form['buttons']['withdraw_button_text_color'] = [
      '#type' => $color_widget,
      '#alpha' => FALSE,
      '#data_theme' => 'polaroid',
      '#size' => 8,
      '#maxlength' => 7,
      '#title' => $this->t('Withdraw consent action button text color'),
      '#default_value' => $this->entity->get('withdraw_button_text_color'),
      '#states' => [
        'visible' => [
          "input[name='enable_withdraw_button']" => ['checked' => TRUE],
          "input[name='formatting']" => ['value' => 'module'],
        ],
      ],
      '#element_validate' => [[$this, 'validateHex']],
    ];
    $form['buttons']['withdraw_button_background_color'] = [
      '#type' => $color_widget,
      '#alpha' => FALSE,
      '#data_theme' => 'polaroid',
      '#size' => 8,
      '#maxlength' => 7,
      '#title' => $this->t('Withdraw consent action button background color'),
      '#default_value' => $this->entity->get('withdraw_button_background_color'),
      '#states' => [
        'visible' => [
          "input[name='enable_withdraw_button']" => ['checked' => TRUE],
          "input[name='formatting']" => ['value' => 'module'],
        ],
      ],
      '#element_validate' => [[$this, 'validateHex']],
    ];
    $form['buttons']['withdraw_button_border_width'] = [
      '#type' => 'textfield',
      '#size' => 15,
      '#maxlength' => 12,
      '#title' => $this->t('Withdraw consent action button border width'),
      '#default_value' => $this->entity->get('withdraw_button_border_width'),
      '#states' => [
        'visible' => [
          "input[name='enable_withdraw_button']" => ['checked' => TRUE],
          "input[name='formatting']" => ['value' => 'module'],
        ],
      ],
    ];
    $form['buttons']['withdraw_button_border_color'] = [
      '#type' => $color_widget,
      '#alpha' => FALSE,
      '#data_theme' => 'polaroid',
      '#size' => 8,
      '#maxlength' => 7,
      '#title' => $this->t('Withdraw consent action button border color'),
      '#default_value' => $this->entity->get('withdraw_button_border_color'),
      '#states' => [
        'visible' => [
          "input[name='enable_withdraw_button']" => ['checked' => TRUE],
          "input[name='formatting']" => ['value' => 'module'],
        ],
      ],
      '#element_validate' => [[$this, 'validateHex']],
    ];
    $form['buttons']['enable_settings_tab'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Enable "Privacy settings" tab'),
      '#description' => $this->t('The "Privacy settings" tab allows for easy opening and closing of the banner.'),
      '#default_value' => $this->entity->get('enable_settings_tab'),
      '#states' => [
        'visible' => [
          [
            "select[name='type']" => [
              [
                'value' => 'banner_at_bottom',
              ],
              [
                'value' => 'banner_at_top',
              ],
              [
                'value' => 'banner_at_top_fixed',
              ],
              [
                'value' => 'side_banner_left',
              ],
              [
                'value' => 'side_banner_right',
              ],
            ],
          ],
        ],
      ],
    ];
    $form['buttons']['settings_tab_label'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Privacy settings tab label'),
      '#default_value' => $this->entity->get('settings_tab_label'),
      '#states' => [
        'visible' => [
          "input[name='enable_settings_tab']" => ['checked' => TRUE],
        ],
      ],
    ];
    $form['buttons']['settings_tab_text_color'] = [
      '#type' => $color_widget,
      '#alpha' => FALSE,
      '#data_theme' => 'polaroid',
      '#size' => 8,
      '#maxlength' => 7,
      '#title' => $this->t('Privacy settings tab text color'),
      '#default_value' => $this->entity->get('settings_tab_text_color'),
      '#states' => [
        'visible' => [
          "input[name='enable_settings_tab']" => ['checked' => TRUE],
          "input[name='formatting']" => ['value' => 'module'],
        ],
      ],
      '#element_validate' => [[$this, 'validateHex']],
    ];
    $form['buttons']['settings_tab_background_color'] = [
      '#type' => $color_widget,
      '#alpha' => FALSE,
      '#data_theme' => 'polaroid',
      '#size' => 8,
      '#maxlength' => 7,
      '#title' => $this->t('Privacy settings tab background color'),
      '#default_value' => $this->entity->get('settings_tab_background_color'),
      '#states' => [
        'visible' => [
          "input[name='enable_settings_tab']" => ['checked' => TRUE],
          "input[name='formatting']" => ['value' => 'module'],
        ],
      ],
      '#element_validate' => [[$this, 'validateHex']],
    ];
    $form['buttons']['settings_tab_border_width'] = [
      '#type' => 'textfield',
      '#size' => 15,
      '#maxlength' => 12,
      '#title' => $this->t('Privacy settings tab border width'),
      '#default_value' => $this->entity->get('settings_tab_border_width'),
      '#states' => [
        'visible' => [
          "input[name='enable_settings_tab']" => ['checked' => TRUE],
          "input[name='formatting']" => ['value' => 'module'],
        ],
      ],
    ];
    $form['buttons']['settings_tab_border_color'] = [
      '#type' => $color_widget,
      '#alpha' => FALSE,
      '#data_theme' => 'polaroid',
      '#size' => 8,
      '#maxlength' => 7,
      '#title' => $this->t('Privacy settings tab border color'),
      '#default_value' => $this->entity->get('settings_tab_border_color'),
      '#states' => [
        'visible' => [
          "input[name='enable_settings_tab']" => ['checked' => TRUE],
          "input[name='formatting']" => ['value' => 'module'],
        ],
      ],
      '#element_validate' => [[$this, 'validateHex']],
    ];
    $form['buttons']['close_button_enabled'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Close button/box enabled'),
      '#description' => $this->t('An X to close the banner.'),
      '#default_value' => $this->entity->get('close_button_enabled'),
    ];
    $form['buttons']['close_button_action'] = [
      '#type' => 'select',
      '#title' => $this->t('Close button action'),
      '#options' => [
        'close_banner' => $this->t('Close banner'),
        'accept_all_cookies' => $this->t('Accept all cookies'),
        'reject_all_cookies' => $this->t('Reject all cookies'),
        'save_preferences' => $this->t('Trigger Save preferences (for banner with categories)'),
      ],
      '#default_value' => $this->entity->get('close_button_action') ?: 'close_banner',
      '#states' => [
        'visible' => [
          "input[name='close_button_enabled']" => ['checked' => TRUE],
        ],
      ],
    ];
    $form['enable_categories_display'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Enable display of categories, for selection'),
      '#default_value' => $this->entity->get('enable_categories_display'),
    ];
    $form['advanced'] = [
      '#type' => 'details',
      '#title' => $this->t('Advanced'),
      '#open' => FALSE,
    ];
    $form['advanced']['containing_element'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Containing element'),
      '#default_value' => $this->entity->get('containing_element'),
    ];
    $form['advanced']['load_at'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Load at'),
      '#default_value' => $this->entity->get('load_at'),
    ];

    // Provide the URL so that we can read it in the Javascript that's set up to
    // assign the ajax handler to the modal form.
    $form['url'] = [
      '#type' => 'hidden',
      '#value' => $this->requestStack->getCurrentRequest()->getPathInfo(),
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function save(array $form, FormStateInterface $form_state) {
    // @todo Clear variables from hidden fields.
    $result = parent::save($form, $form_state);
    $message_args = ['%label' => $this->entity->label()];
    $message = $result == SAVED_NEW
      ? $this->t('Created new cookie banner/dialog %label.', $message_args)
      : $this->t('Updated cookie banner/dialog %label.', $message_args);
    $this->messenger()->addStatus($message);
    $form_state->setRedirectUrl($this->entity->toUrl('collection'));
    return $result;
  }

  /**
   * Validate field for a HEX value if a value is set.
   *
   * @param array $element
   *   Element.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   Form State Interface.
   */
  public function validateHex(array $element, FormStateInterface $form_state) {
    if (!empty($element['#value']) && !preg_match('/^#[0-9a-fA-F]{3,6}$/', $element['#value'])) {
      $form_state->setError($element, $this->t('%name must be a HEX value or empty.', ['%name' => $element['#title']]));
    }
  }

}
