<?php

namespace Drupal\eu_cookie_compliance\Form;

use Drupal\Component\Serialization\Json;
use Drupal\Component\Utility\UrlHelper;
use Drupal\Core\Entity\EntityForm;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Extension\ModuleHandlerInterface;
use Drupal\Core\Form\FormBuilderInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Language\LanguageManagerInterface;
use Drupal\Core\Site\Settings;
use Drupal\Core\Url;
use Drupal\eu_cookie_compliance\Plugin\ConsentStorageManager;
use Drupal\eu_cookie_compliance\RegionHandler;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Cookie Compliance Set form.
 *
 * @property \Drupal\eu_cookie_compliance\CookieComplianceSetInterface $entity
 */
class CookieComplianceSetForm extends EntityForm {

  /**
   * Entity Type Manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * Form Builder.
   *
   * @var \Drupal\Core\Form\FormBuilderInterface
   */
  protected $formBuilder;

  /**
   * The module handler.
   *
   * @var \Drupal\Core\Extension\ModuleHandlerInterface
   */
  protected $moduleHandler;

  /**
   * The consent storage.
   *
   * @var \Drupal\eu_cookie_compliance\Plugin\ConsentStorageManagerInterface
   */
  protected $consentStorage;

  /**
   * The consent storage.
   *
   * @var \Drupal\eu_cookie_compliance\RegionHandler
   */
  protected $regionHandler;

  /**
   * The language manager.
   *
   * @var \Drupal\Core\Language\LanguageManagerInterface
   */
  protected $languageManager;

  /**
   * Creates a CookieComplianceSetForm instance.
   *
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   Entity Type manager.
   * @param \Drupal\Core\Form\FormBuilderInterface $form_builder
   *   Form builder.
   * @param \Drupal\Core\Extension\ModuleHandlerInterface $module_handler
   *   The module handler.
   * @param \Drupal\eu_cookie_compliance\Plugin\ConsentStorageManager $consent_storage
   *   EU Cookie Compliance Consent Storage.
   * @param \Drupal\eu_cookie_compliance\RegionHandler $region_handler
   *   EU Cookie Compliance Region Handler.
   * @param \Drupal\Core\Language\LanguageManagerInterface $language_manager
   *   Language Manager.
   */
  public function __construct(EntityTypeManagerInterface $entity_type_manager, FormBuilderInterface $form_builder, ModuleHandlerInterface $module_handler, ConsentStorageManager $consent_storage, RegionHandler $region_handler, LanguageManagerInterface $language_manager) {
    $this->entityTypeManager = $entity_type_manager;
    $this->formBuilder = $form_builder;
    $this->moduleHandler = $module_handler;
    $this->consentStorage = $consent_storage;
    $this->regionHandler = $region_handler;
    $this->languageManager = $language_manager;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('entity_type.manager'),
      $container->get('form_builder'),
      $container->get('module_handler'),
      $container->get('plugin.manager.eu_cookie_compliance.consent_storage'),
      $container->get('eu_cookie_compliance.region_handler'),
      $container->get('language_manager')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function form(array $form, FormStateInterface $form_state) {

    // @todo Allow selection of breakpoints (mobile/desktop).
    $form = parent::form($form, $form_state);

    $form['#attached']['library'][] = 'core/drupal.dialog.ajax';
    $form['#attached']['library'][] = 'eu_cookie_compliance/ajax_commands';

    $banners = $this->getList($this->entityTypeManager->getStorage('cookie_banner')->loadMultiple(), 'cookie-banners');

    // List of checkbox values.
    $role_names = [];
    // Permissions per role.
    $role_permissions = [];
    // Which checkboxes should be ticked.
    $role_values = [];

    $perm = 'display eu cookie compliance banner ' . $this->entity->id();

    foreach ($this->entityTypeManager->getStorage('user_role')->loadMultiple() as $role_name => $role) {
      // Exclude Admin roles.
      /** @var \Drupal\user\Entity\Role $role */
      if (!$role->isAdmin()) {
        $role_names[$role_name] = $role->label();
        // Fetch permissions for the roles.
        $role_permissions[$role_name] = $role->getPermissions();
        // Indicate whether the checkbox should be ticked.
        if (in_array($perm, $role_permissions[$role_name]) || $this->entity->id() === NULL) {
          $role_values[] = $role_name;
        }
      }
    }

    $form['label'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Name'),
      '#maxlength' => 255,
      '#default_value' => $this->entity->label(),
      '#description' => $this->t('Administrative name for the cookie compliance set.'),
      '#required' => TRUE,
    ];

    $form['id'] = [
      '#type' => 'machine_name',
      '#default_value' => $this->entity->id(),
      '#machine_name' => [
        'exists' => '\Drupal\eu_cookie_compliance\Entity\CookieComplianceSet::load',
      ],
      '#disabled' => !$this->entity->isNew(),
    ];

    $form['enabled'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Enable banner'),
      '#default_value' => $this->entity->id() === NULL ? TRUE : $this->entity->get('enabled'),
    ];

    $form['permissions'] = [
      '#type' => 'details',
      '#title' => $this->t('Permissions'),
      '#open' => TRUE,
    ];

    $form['permissions']['display_banner_for_roles'] = [
      '#type' => 'checkboxes',
      '#title' => $this->t('Display for'),
      '#options' => $role_names,
      '#default_value' => $role_values ?: [],
    ];

    $form['consent_method'] = [
      '#type' => 'details',
      '#title' => $this->t('Consent for processing of personal information'),
      '#open' => TRUE,
    ];

    $form['consent_method']['method'] = [
      '#type' => 'radios',
      '#title' => $this->t('Method'),
      '#options' => [
        'default' => $this->t("Informational. Don't provide any option to opt in or out."),
        'opt_in' => $this->t("Opt-in. Don't track visitors unless they specifically give consent. (GDPR compliant)"),
        'categories' => $this->t('Opt-in with categories. Let visitors choose which cookie categories they want to opt-in for (GDPR compliant).'),
        'opt_out' => $this->t('Opt-out. Track visitors by default, unless they choose to opt out.'),
      ],
      '#default_value' => $this->entity->get('method') ?: 'opt_in',
    ];

    $form['cookie_banners_group'] = [
      '#type' => 'details',
      '#title' => $this->t('Cookie Banners/Dialogs'),
      '#open' => TRUE,
    ];

    // @todo Send the method as a parameter to the Edit / Duplicate dialog.
    $form['cookie_banners_group']['cookie_banners'] = [
      '#title' => $this->t('Enabled'),
      '#type' => 'checkboxes',
      '#options' => $banners,
      '#default_value' => $this->entity->get('cookie_banners') ?: [],
    ];

    $form['cookie_banners_group']['add_banner'] = [
      '#title' => $this->t('Add cookie banner/dialog'),
      '#type' => 'link',
      '#url' => Url::fromUserInput('/eu-cookie-compliance/add-cookie-banner'),
      '#attributes' => [
        'class' => ['use-ajax', 'button'],
        'data-dialog-type' => 'modal',
        'data-dialog-options' => JSON::encode([
          'width' => 640,
        ]),
      ],
    ];

    $regions = $this->regionHandler->getSortedRegions();

    $form['regions_group'] = [
      '#type' => 'details',
      '#title' => $this->t('Regions'),
      '#open' => FALSE,
    ];

    if ($this->moduleHandler->moduleExists('smart_ip') || extension_loaded('geoip')) {

      $form['regions_group']['info'] = [
        '#type' => 'markup',
        '#markup' => $this->t('<p>Select from the lists below to limit the number of countries where the banners are displayed. This list was complete as of Oct 1, 2022 and includes all countries that have some kind of legislation that affects cookies. Not all of the countries listed below require opt-in consent. Some require opt-out functionality and others require only an informational banner. Please help keep this list up to date. If you find a country that has cookie legislation and is not in this list, it would be great if you can <a href="https://www.drupal.org/node/add/project-issue/eu_cookie_compliance">open an issue on drupal.org to bring that to our attention</a>. Please include a link to the article you read or the law itself.  Using the <a href="http://drupal.org/project/smart_ip">smart_ip</a> module or the <a href="http://www.php.net/manual/en/function.geoip-country-code-by-name.php">geoip_country_code_by_name()</a> PHP function.</p>'),
      ];

      foreach ($regions as $continent => $countries) {
        $options = [];
        foreach ($countries as $country_key => $country) {
          $options[$country_key] = $country['name'];
        }
        $form['regions_group'][strtolower(str_replace(' ', '_', $continent))] = [
          '#type' => 'checkboxes',
          '#options' => $options,
          '#title' => $continent,
          '#default_value' => $this->entity->get('regions') ?: [],
        ];
      }
      $form['regions_group']['custom_regions_info'] = [
        '#type' => 'checkbox',
        '#description' => $this->t('Write a list of ISO-3166 alpha 2 country codes that will complement any selection done in the lists above.'),
        '#title' => $this->t('Custom'),
        '#default_value' => $this->entity->get('custom_regions') !== '',
      ];

      $form['regions_group']['custom_regions'] = [
        '#type' => 'textfield',
        '#default_value' => $this->entity->get('custom_regions'),
        '#states' => [
          'visible' => [
            "input[name='custom_regions_info']" => ['checked' => TRUE],
          ],
        ],
        '#description' => $this->t('Provide a comma separated list. Countries should be given as 2 character codes. States in the USA should be written as US/CA for California, where CA is the postal code.'),
      ];
    }
    else {
      $form['regions_group']['info'] = [
        '#markup' => $this->t('You can choose to show the banner only to visitors from certain countries. In order to achieve this, you need to install the <a href="http://drupal.org/project/smart_ip">smart_ip</a> module or enable the <a href="http://www.php.net/manual/en/function.geoip-country-code-by-name.php">geoip_country_code_by_name()</a> PHP function.'),
      ];
    }

    $languages = $this->getEnabledLanguages();

    if (count($languages) > 1) {
      $form['languages_group'] = [
        '#type' => 'details',
        '#title' => $this->t('Languages'),
        '#open' => TRUE,
        '#description' => $this->t("Select the languages that this config set should be used for. If you've enabled regions in the setting above, the banner will be shown if BOTH the region AND the language matches the one(s) selected."),
      ];

      $form['languages_group']['languages'] = [
        '#type' => 'checkboxes',
        '#options' => $languages,
        '#default_value' => $this->entity->get('languages') ?: [],
      ];
    }

    if ($this->moduleHandler->moduleExists('domain')) {
      $domains = $this->entityTypeManager->getStorage('domain')->loadMultiple();
      $domains_list = [];
      foreach ($domains as $entity) {
        $domains_list[$entity->id()] = $entity->label();
      }

      if (count($languages) > 1) {
        $form['domains_group'] = [
          '#type' => 'details',
          '#title' => $this->t('Domains'),
          '#open' => TRUE,
        ];

        $form['domains_group']['domains'] = [
          '#type' => 'checkboxes',
          '#options' => $domains_list,
          '#default_value' => $this->entity->get('domains') ?: [],
        ];
      }
    }

    $form['privacy_policy'] = [
      '#type' => 'details',
      '#title' => $this->t('Privacy policy'),
      '#open' => TRUE,
    ];

    $form['privacy_policy']['privacy_policy_link'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Privacy policy link'),
      '#default_value' => $this->entity->get('privacy_policy_link'),
      '#maxlength' => 1024,
      '#required' => TRUE,
      '#description' => $this->t('Enter link to your privacy policy or other page that will explain cookies to your users, external links should start with http:// or https://.'),
      '#element_validate' => [[$this, 'validatePrivacyPolicyLink']],
    ];

    $form['privacy_policy']['open_privacy_link_in_new_window'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Open the privacy policy in a new window.'),
      '#default_value' => $this->entity->get('open_privacy_link_in_new_window'),
    ];

    $form['privacy_policy']['enable_policy_versioning'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Enable policy versioning.'),
      '#default_value' => $this->entity->get('enable_policy_versioning') === NULL ? TRUE : $this->entity->get('enable_policy_versioning'),
    ];

    $form['privacy_policy']['privacy_policy_version'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Privacy policy version'),
      '#description' => $this->t('Change this value to make the cookie information banner reappear for all visitors. A typical usage of this field is to increase the value when the privacy policy has been changed.'),
      '#default_value' => $this->entity->get('privacy_policy_version') ?: '1.0.0',
      '#required' => TRUE,
      '#states' => [
        'visible' => [
          "input[name='enable_policy_versioning']" => ['checked' => TRUE],
        ],
      ],
    ];

    $form['cookie'] = [
      '#type' => 'details',
      '#title' => $this->t('Cookie'),
      '#open' => FALSE,
    ];

    $form['cookie']['cookie_name'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Cookie name'),
      '#default_value' => $this->entity->get('cookie_name') ?: '',
      '#description' => $this->t('Sets the cookie name that is used to check whether the user has agreed or not.'),
      '#element_validate' => [[$this, 'validateCookieName']],
    ];

    $form['cookie']['cookie_same_site'] = [
      '#type' => 'select',
      '#options' => [
        'lax' => $this->t('Lax'),
        'strict' => $this->t('Strict'),
        'none' => $this->t('None'),
      ],
      '#title' => $this->t('Same Site'),
      '#default_value' => $this->entity->get('cookie_same_site') ?: 'strict',
      '#description' => $this->t('The SameSite attribute of the Set-Cookie HTTP response header allows you to declare if your cookie should be restricted to a first-party or same-site context.'),
    ];

    $form['cookie']['cookie_value_disagreed'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Cookie value (disagreed)'),
      '#default_value' => $this->entity->get('cookie_value_disagreed') ?: '0',
      '#description' => $this->t('The cookie value which will be set where the user has disagreed.'),
    ];

    $form['cookie']['cookie_value_agreed'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Cookie value (agreed)'),
      '#default_value' => $this->entity->get('cookie_value_agreed') ?: '2',
      '#description' => $this->t('The cookie value which will be set where the user has agreed.'),
    ];

    $form['cookie']['cookie_lifetime'] = [
      '#type' => 'number',
      '#title' => $this->t('Cookie lifetime'),
      '#description' => $this->t("How many days the system remember the user's choice."),
      '#default_value' => $this->entity->get('cookie_lifetime') ?: 100,
      '#field_suffix' => $this->t('days'),
      '#size' => 5,
      '#maxlength' => 5,
      '#required' => TRUE,
      '#states' => [
        'enabled' => [
          "input[name='cookie_session']" => ['checked' => FALSE],
        ],
      ],
    ];

    $form['javascripts'] = [
      '#type' => 'details',
      '#title' => $this->t("Disable the following JavaScripts when consent isn't given"),
      '#open' => TRUE,
      '#states' => [
        'visible' => [
          "input[name='method']" => ['!value' => 'default'],
        ],
      ],
    ];

    $form['javascripts']['disabled_javascripts'] = [
      '#type' => 'textarea',
      '#title' => $this->t('Disable JavaScripts'),
      '#default_value' => $this->entity->get('disabled_javascripts'),
      '#description' => $this->t("<span class='eu-cookie-compliance-word-break'>Include the full path of JavaScripts, each on a separate line. When using the opt-in or opt-out consent options, you can block certain JavaScript files from being loaded when consent isn't given. The on-site JavaScripts should be written as root relative paths <strong>without the leading slash</strong>, you can use public://path/to/file.js and private://path/to/file.js, and off-site JavaScripts should be written as complete URLs <strong>with the leading http(s)://</strong>. Note that after the user gives consent, the scripts will be executed in the order you enter here.<br /><br />Libraries and scripts that attach to Drupal.behaviors are supported. To indicate a behavior that needs to be loaded on consent, append the behavior name after the script with a | (vertical bar). If you also want to conditionally load a library, place that as the third parameter, following another | (vertical bar). <strong>Example: modules/custom/custom_module/js/custom.js|customModule|custom_module/custom_module</strong>.<br />If your script file does not attach to Drupal.attributes, you may skip the second parameter. <strong>Example: modules/custom/custom_module/js/custom.js||custom_module/custom_module</strong><br /><strong>Note that Drupal behavior name and library parameters are both optional</strong>, but may be required to achieve your objective.</span>") .
        '<br /><br />' . $this->t('When using the consent method "Opt-in with categories", you can link the script to a specific category by using the format: "category:path/to/the/script.js".'),
    ];

    $form['cookies'] = [
      '#type' => 'details',
      '#title' => $this->t('Cookie handling'),
      '#open' => TRUE,
      '#states' => [
        'visible' => [
          "input[name='method']" => ['!value' => 'default'],
        ],
      ],
    ];

    $form['cookies']['automatic_cookies_removal'] = [
      '#type' => 'checkbox',
      '#title' => $this->t("Enable cookie(s) automatic-removal when consent isn't given."),
      '#default_value' => !empty($this->entity->get('automatic_cookies_removal')) ? $this->entity->get('automatic_cookies_removal') : FALSE,
    ];

    $form['cookies']['allowed_cookies'] = [
      '#type' => 'textarea',
      '#title' => $this->t('Allowed cookies'),
      '#default_value' => $this->entity->get('"allowed_cookies"'),
      '#description' => $this->t("Include the name of cookies, each on a separate line. When using the opt-in or opt-out consent options, this module will <strong>delete cookies from your domain that are not allowed</strong> every few seconds when consent isn't given. PHP session cookies and the cookie for this module are always allowed.") .
        '<br /><br />' . $this->t('When using the consent method "Opt-in with categories", you can link the cookie to a specific consent category by using the format: "category:cookie_name".  Only when consent is given for the given category, will the cookie be allowed.') .
        '<br />' . $this->t('Cookie names can contain "*" characters which mean a series of any characters.'),
      '#states' => ['visible' => ['input[name="automatic_cookies_removal"]' => ['checked' => TRUE]]],
    ];

    $form['consent_storage'] = [
      '#type' => 'details',
      '#title' => $this->t('Store record of consent'),
      '#open' => FALSE,
      '#states' => [
        'visible' => [
          "input[name='method']" => ['!value' => 'default'],
        ],
      ],
    ];

    $form['consent_storage']['info'] = [
      '#type' => 'markup',
      '#markup' => $this->t('Depending on your implementation of GDPR, you may have to store a record when the user consents. This module comes with a basic consent storage plugin that writes a record to the database. Note that if your site has significant traffic, the basic consent storage may become a bottleneck, as every consent action will require a write to the database. You can easily create your own module with a ConsentStorage Plugin that extends ConsentStorageBase, using BasicConsentStorage from this module as a template. If you create a highly performant consent storage plugin, please consider contributing it back to the Drupal community as a contrib module.'),
    ];

    $plugin_definitions = $this->consentStorage->getDefinitions();

    $consent_storage_options = [];
    $consent_storage_options['do_not_store'] = $this->t('Do not store');
    foreach ($plugin_definitions as $plugin_name => $plugin_definition) {
      /** @var \Drupal\Core\StringTranslation\TranslatableMarkup $plugin_definition_name */
      $plugin_definition_name = $plugin_definition['name'];
      $consent_storage_options[$plugin_name] = $plugin_definition_name->render();
    }

    $form['consent_storage']['consent_storage_method'] = [
      '#type' => 'radios',
      '#title' => $this->t('Consent storage method'),
      '#default_value' => $this->entity->get('consent_storage_method') ?: 'do_not_store',
      '#options' => $consent_storage_options,
    ];

    $form['advanced'] = [
      '#type' => 'details',
      '#open' => FALSE,
      '#title' => $this->t('Advanced'),
    ];

    // Adding option to add/remove banner on specified domains.
    $exclude_domains_option_active = [
      'add' => $this->t('Add'),
      'remove' => $this->t('Remove'),
    ];

    $form['advanced']['consent_by_clicking'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Consent by clicking (Not GDPR compliant).'),
      '#default_value' => $this->entity->get('consent_by_clicking') ?: FALSE,
    ];

    $form['advanced']['add_remove_banner_on_domain'] = [
      '#type' => 'radios',
      '#title' => $this->t('Add/remove banner on specified domains'),
      '#default_value' => $this->entity->get('add_remove_banner_on_domain') ?: 'remove',
      '#options' => $exclude_domains_option_active,
      '#description' => $this->t('Specify if you want to add or remove banner on the listed below domains.'),
    ];

    $form['advanced']['add_remove_domains'] = [
      '#type' => 'textarea',
      '#title' => $this->t('Domains list'),
      '#default_value' => $this->entity->get('add_remove_domains') ?: '',
      '#description' => $this->t('Specify domains with protocol (e.g., http or https). Enter one domain per line.'),
    ];

    $form['advanced']['exclude_paths'] = [
      '#type' => 'textarea',
      '#title' => $this->t('Exclude paths'),
      '#default_value' => $this->entity->get('exclude_paths') ?: '',
      '#description' => $this->t("Specify pages by using their paths. Enter one path per line. The '*' character is a wildcard. Example paths are %blog for the blog page and %blog-wildcard for every personal blog. %front is the front page.", [
        '%blog' => '/blog',
        '%blog-wildcard' => '/blog/*',
        '%front' => '<front>',
      ]),
    ];

    $form['advanced']['exclude_admin_theme'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Exclude admin pages.'),
      '#default_value' => $this->entity->get('exclude_admin_theme') ?: FALSE,
      '#description' => $this->t('Note: if you enable the Withdraw feature, the Privacy settings tab will be displayed also on admin pages, but the banner will not open until you click the tab.'),
    ];

    $form['advanced']['exclude_uid_1'] = [
      '#type' => 'checkbox',
      '#title' => $this->t("Don't show the banner for site administrators (including UID 1)."),
      '#default_value' => $this->entity->get('exclude_uid_1') ?: 0,
      '#description' => $this->t('Note: if you enable the Withdraw feature, the Privacy settings tab will be displayed also for site administrators, but the banner will not open until you click the tab.'),
    ];

    $form['advanced']['prompt_at_every_session'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Prompt for consent (from the same user) at every new browser session.'),
      '#description' => $this->t("This sets cookie lifetime to 0, invalidating the cookie at the end of the browser session. To set a cookie lifetime greater than 0, uncheck this option. Note that some users will find this behavior highly annoying, and it's recommended to double-check with the legal advisor whether you really need this option enabled."),
      '#default_value' => $this->entity->get('prompt_at_every_session') ?: FALSE,
    ];

    $form['advanced']['domain'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Domain'),
      '#default_value' => $this->entity->get('domain') ?: '',
      '#description' => $this->t('Sets the domain of the cookie to a specific url. Used when you need consistency across domains. This is language independent. Note: Make sure you actually enter a domain that the browser can make use of. For example if your site is accessible at both www.domain.com and domain.com, you will not be able to hide the banner at domain.com if your value for this field is www.domain.com.'),
    ];

    $form['advanced']['set_cookie_for_all_domains'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Allow the cookie to be set for all sites on the same domain.'),
      '#default_value' => $this->entity->get('set_cookie_for_all_domains') ?: FALSE,
      '#description' => $this->t("Sets the path of the cookie to '/' so that the cookie works across all sites on the domain."),
    ];

    $form['advanced']['reload_page'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Reload page after user clicks the "Agree" button.'),
      '#default_value' => $this->entity->get('reload_page') ?: 0,
    ];

    // Adding option to include/exclude routes to reload.
    $reload_options = [
      'include' => $this->t('Include'),
      'exclude' => $this->t('Exclude'),
    ];

    $form['advanced']['reload_options'] = [
      '#type' => 'radios',
      '#title' => $this->t('Include/exclude specified routes for reloading'),
      '#default_value' => $this->entity->get('reload_options') ?: 'include',
      '#options' => $reload_options,
      '#description' => $this->t('Specify if you want to include or exclude the listed below routes for reloading the page after agreeing cookies.'),
      '#states' => [
        'visible' => [
          ['input[name="reload_page"]' => ['checked' => TRUE]],
        ],
      ],
    ];

    $form['advanced']['reload_routes_list'] = [
      '#type' => 'textarea',
      '#title' => $this->t('Routes list'),
      '#default_value' => $this->entity->get('reload_routes_list') ?: '',
      '#description' => $this->t("Specify pages by using their paths. Enter one path per line. The '*' character is a wildcard. Example paths are %blog for the blog page and %blog-wildcard for every personal blog. Leave blank for reloading all pages. Leave the list empty to reload on all pages. This field does not support url aliases.", [
        '%blog' => '/blog',
        '%blog-wildcard' => '/blog/*',
      ]),
      '#states' => [
        'visible' => [
          ['input[name="reload_page"]' => ['checked' => TRUE]],
        ],
      ],
    ];

    $form['advanced']['set_cookie_session_zero_on_disagree'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Set cookie lifetime to 0 if the user clicks the "Decline" or the "Withdraw consent" button.'),
      '#description' => $this->t('Enable this if you want to show the consent banner at every new session until the user clicks the "Agree" button.'),
      '#default_value' => $this->entity->get('set_cookie_session_zero_on_disagree') ?: FALSE,
    ];

    if ($this->moduleHandler->moduleExists('coloris')) {
      $form['#attached']['library'][] = 'coloris/element.coloris';
    }

    $form['#attached']['library'][] = 'eu_cookie_compliance/admin.cookie-banner';

    return $form;
  }

  /**
   * Get all regions.
   *
   * @return array
   *   The regions.
   */
  protected function getList(array $entities, $machine_name = '') : array {
    $set = [];
    foreach ($entities as $entity) {
      $id = $entity->id();
      $set[$id] = $entity->label() . ' <a href="/eu-cookie-compliance/edit-' . $machine_name . '/' . $id . '" class="use-ajax" data-dialog-type="modal" data-dialog-options="{&quot;width&quot;: 640}">Edit</a>';
    }
    return $set;
  }

  /**
   * Get all enabled languages.
   *
   * @return array
   *   The enabled languages.
   */
  protected function getEnabledLanguages() {
    $languages = $this->languageManager->getLanguages();
    $processed_languages = [];
    foreach ($languages as $key => $language) {
      $processed_languages[$key] = $language->getName();
    }
    return $processed_languages;
  }

  /**
   * {@inheritdoc}
   */
  public function save(array $form, FormStateInterface $form_state) {
    // If custom regions is not enabled, cancel out any value in the setting.
    if ($form_state->getValue('custom_regions_info') === 0) {
      $this->entity->set('custom_regions', '');
    }

    // Process the selected regions.
    $continents = $this->regionHandler->getSortedRegions();
    $regions = [];
    foreach ($continents as $continent => $values) {
      $countries = $form_state->getValue(strtolower(str_replace(' ', '_', $continent)));
      if ($countries) {
          foreach ($countries as $country => $value) {
              if ($value !== 0) {
                  $regions[] = $country;
              }
          }
      }
    }
    $this->entity->set('regions', $regions);

    // Remove unselected cookie banners.
    $cookie_banners = $form_state->getValue('cookie_banners');
    foreach ($cookie_banners as $key => $cookie_banner) {
      if ($cookie_banner === 0) {
        unset($cookie_banners[$key]);
      }
    }
    $this->entity->set('cookie_banners', $cookie_banners);

    // Save permissions.
    $permission_name = 'display eu cookie compliance banner ' . $this->entity->id();

    $readonly_enabled = $this->moduleHandler->moduleExists('config_readonly') && Settings::get('config_readonly');
    $allowlist_patterns = Settings::get('config_readonly_whitelist_patterns');

    $roles = $this->entityTypeManager->getStorage('user_role')->loadMultiple();
    // Save permissions.
    foreach ($roles as $role_name => $role) {
      /** @var \Drupal\user\Entity\Role $role */
      if (!$role->isAdmin()) {
        $user_settings_allowlisted = !empty($allowlist_patterns) && in_array('user.role.' . $role_name, $allowlist_patterns);
        if (!$readonly_enabled || ($readonly_enabled && $user_settings_allowlisted)) {
          if (array_key_exists($role_name, $form_state->getValue('display_banner_for_roles')) && $form_state->getValue('display_banner_for_roles')[$role_name]) {
            $role->grantPermission($permission_name);
          }
          else {
            $role->revokePermission($permission_name);
          }
          $role->trustData()->save();
        }
        else {
          $this->messenger()->addWarning($this->t('The user role permission could not be set because the <code>config_readonly</code> module is enabled. To allow setting of user role permissions, please add <code>user.role.%name</code> to the <code>config_readonly</code> allowlist.', ['%name' => $role_name]));
        }
      }
    }

    $result = parent::save($form, $form_state);
    $message_args = ['%label' => $this->entity->label()];
    $message = $result == SAVED_NEW
      ? $this->t('Created new cookie compliance set %label.', $message_args)
      : $this->t('Updated cookie compliance set %label.', $message_args);
    $this->messenger()->addStatus($message);
    $form_state->setRedirectUrl($this->entity->toUrl('collection'));
    return $result;
  }

  /**
   * Validates the banner link field.
   *
   * @param array $element
   *   Element.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   Form state.
   */
  public function validatePrivacyPolicyLink(array $element, FormStateInterface $form_state) {
    if (empty($element['#value'])) {
      return;
    }

    $input = $element['#value'];
    if (UrlHelper::isExternal($input)) {
      $allowed_protocols = ['http', 'https'];
      if (!in_array(parse_url($input, PHP_URL_SCHEME), $allowed_protocols)) {
        $form_state->setError($element, $this->t('Invalid protocol specified for the %name (valid protocols: %protocols).', [
          '%name' => $element['#title'],
          '%protocols' => implode(', ', $allowed_protocols),
        ]));
      }
      else {
        try {
          Url::fromUri($input);
        }
        catch (\Exception $exc) {
          $form_state->setError($element, $this->t('Invalid %name (:message).', [
            '%name' => $element['#title'],
            ':message' => $exc->getMessage(),
          ]));
        }
      }
    }
    else {
      // Special case for '<front>'.
      if ($input === '<front>') {
        $input = '/';
      }
      try {
        Url::fromUserInput($input);
      }
      catch (\Exception $exc) {
        $form_state->setError($element, $this->t('Invalid URL in %name field (:message).', [
          '%name' => $element['#title'],
          ':message' => $exc->getMessage(),
        ]));
      }
    }
  }

  /**
   * Validates the cookie name.
   *
   * @param array $element
   *   Element.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   Form state.
   */
  public function validateCookieName(array $element, FormStateInterface $form_state) {
    if (empty($element['#value'])) {
      return;
    }

    $input = $element['#value'];
    // Validate cookie name against valid characters.
    if (preg_match('/[&\'\x00-\x20\x22\x28-\x29\x2c\x2f\x3a-\x40\x5b-\x5d\x7b\x7d\x7f]/', $input)) {
      $form_state->setError($element, $this->t('Invalid cookie name, please remove any special characters and try again.'));
    }
  }

}
