<?php

namespace Drupal\eu_cookie_compliance;

use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Extension\ModuleHandlerInterface;

/**
 * Service that handles default cookie compliance regions.
 */
class RegionHandler {

  /**
   * The module handler service.
   *
   * @var \Drupal\Core\Extension\ModuleHandlerInterface
   */
  protected $moduleHandler;

  /**
   * The entity type manager service.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * Constructs a RegionHandler object.
   *
   * @param \Drupal\Core\Extension\ModuleHandlerInterface $module_handler
   *   Module Handler.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   Entity Type Manager.
   */
  public function __construct(ModuleHandlerInterface $module_handler, EntityTypeManagerInterface $entity_type_manager) {
    $this->moduleHandler = $module_handler;
    $this->entityTypeManager = $entity_type_manager;
  }

  /**
   * Method description.
   */
  public function getRegions() : array {
    $cookie_compliance_regions = [
      'eu' => [
        'name' => 'European Union and European Economic Area - General Data Protection Regulation (GDPR)',
        'regions' => [
          'AT', 'AX', 'BE', 'BG', 'CY', 'CZ', 'DE', 'DK', 'EE', 'EL', 'ES',
          'EU', 'FI', 'FR', 'GB', 'GF', 'GI', 'GP', 'GR', 'HR', 'HU', 'IE', 'IS',
          'IT', 'LI', 'LT', 'LU', 'LV', 'ME', 'MF', 'MQ', 'MT', 'NL', 'NO', 'PL',
          'PT', 'RE', 'RO', 'SE', 'SI', 'SJ', 'SK', 'YT',
        ],
        'continent' => 'EU',
      ],
      'uk' => [
        'name' => 'United Kingdom of Great Britain and Northern Ireland - The U.K. Data Protection Act',
        'regions' => ['UK'],
        'continent' => 'Europe (non-EU)',
      ],
      'bosnia_and_herzegovina' => [
        'name' => 'Bosnia and Herzegovina - Law on protection of Personal Data (LPPD)',
        'regions' => ['BA'],
        'continent' => 'Europe (non-EU)',
      ],
      'serbia' => [
        'name' => 'Serbia - Law on Protection of Personal Data (LPPD)',
        'regions' => ['RS'],
        'continent' => 'Europe (non-EU)',
      ],
      'russia' => [
        'name' => 'Russia - Personal Data Law',
        'regions' => ['RU'],
        'continent' => 'Europe (non-EU)',
      ],
      'andorra' => [
        'name' => 'Andorra - Law of Personal Data Protection (LPDR)',
        'regions' => ['AD'],
        'continent' => 'Europe (non-EU)',
      ],
      'albania' => [
        'name' => 'Albania - Law on the Protection of Personal Data (LPDR)',
        'regions' => ['AL'],
        'continent' => 'Europe (non-EU)',
      ],
      'faroe_islands' => [
        'name' => 'Faroe Islands - Act on the Protection of Personal Data',
        'regions' => ['FO'],
        'continent' => 'Europe (non-EU)',
      ],
      'guernsey' => [
        'name' => 'Guernsey - Data Protection Law, 2017, as amended',
        'regions' => ['GG'],
        'continent' => 'Europe (non-EU)',
      ],
      'isle_of_man' => [
        'name' => 'Isle of Man - Data Protection Act',
        'regions' => ['IM'],
        'continent' => 'Europe (non-EU)',
      ],
      'jersey' => [
        'name' => 'Jersey - Data Protection Law',
        'regions' => ['JE'],
        'continent' => 'Europe (non-EU)',
      ],
      'monaco' => [
        'name' => 'Monaco - Bill No. 1054 Relating to the Protection of Personal Data',
        'regions' => ['MC'],
        'continent' => 'Europe (non-EU)',
      ],
      'moldova' => [
        'name' => 'Moldova - Law No. 133 on Personal Data Protection',
        'regions' => ['MD'],
        'continent' => 'Europe (non-EU)',
      ],
      'montenegro' => [
        'name' => 'Montenegro - Personal Data Protection Law',
        'regions' => ['ME'],
        'continent' => 'Europe (non-EU)',
      ],
      'north_macedonia' => [
        'name' => 'North Macedonia - Law on Personal Data Protection',
        'regions' => ['MK'],
        'continent' => 'Europe (non-EU)',
      ],
      'montserrat' => [
        'name' => 'Montserrat - General Data Protection Regulation (GDPR)',
        'regions' => ['MS'],
        'continent' => 'North America',
      ],
      'malta' => [
        'name' => 'Malta - General Data Protection Regulation (GDPR)',
        'regions' => ['MT'],
        'continent' => 'Europe (non-EU)',
      ],
      'gibraltar' => [
        'name' => 'Gibraltar - Gibraltar General Data Protection Regulation',
        'regions' => ['GI'],
        'continent' => 'Europe (non-EU)',
      ],
      'greenland' => [
        'name' => 'Greenland - Personal Data Protection Act',
        'regions' => ['GL'],
        'continent' => 'Europe (non-EU)',
      ],
      'belarus' => [
        'name' => 'Belarus - Law on Personal Data Protection (LPDP)',
        'regions' => ['BY'],
        'continent' => 'Europe (non-EU)',
      ],
      'ukraine' => [
        'name' => 'Ukraine - Draft Law “On Personal Data Protection” No. 5628',
        'regions' => ['UA'],
        'continent' => 'Europe (non-EU)',
      ],
      'kosovo' => [
        'name' => 'Kosovo - Law No.03/L - 172 on the Protection of Personal Data',
        'regions' => ['XK'],
        'continent' => 'Europe (non-EU)',
      ],
      'united_arab_emirates' => [
        'name' => 'United Arab Emirates - Personal Data Protection Law (PDPL)',
        'regions' => ['AE'],
        'continent' => 'Asia',
      ],
      'sri_lanka' => [
        'name' => 'Sri Lanka -  Personal Data Protection Act',
        'regions' => ['LK'],
        'continent' => 'Asia',
      ],
      'israel' => [
        'name' => 'Israel - Protection of Privacy Law',
        'regions' => ['IL'],
        'continent' => 'Asia',
      ],
      'mongolia' => [
        'name' => 'Mongolia - Law of Mongolia on Personal Data Protection (PDPA)',
        'regions' => ['MN'],
        'continent' => 'Asia',
      ],
      'malaysia' => [
        'name' => 'Malaysia - Personal Data Protection Act',
        'regions' => ['MY'],
        'continent' => 'Asia',
      ],
      'qatar' => [
        'name' => 'Qatar - Law No. (13) of 2016 Concerning Personal Data Protection',
        'regions' => ['QA'],
        'continent' => 'Asia',
      ],
      'saudi_arabia' => [
        'name' => 'Saudi Arabia - Personal Data Protection Law',
        'regions' => ['SA'],
        'continent' => 'Asia',
      ],
      'singapore' => [
        'name' => 'Singapore - Personal Data Protection Act (PDPA)',
        'regions' => ['SG'],
        'continent' => 'Asia',
      ],
      'thailand' => [
        'name' => 'Thailand - Personal Data Protection Act (PDPA)',
        'regions' => ['SG'],
        'continent' => 'Asia',
      ],
      'tajikistan' => [
        'name' => 'Tajikistan - Personal Data Protection Law, No.1537',
        'regions' => ['TJ'],
        'continent' => 'Asia',
      ],
      'turkmenistan' => [
        'name' => 'Turkmenistan - Law of Turkmenistan of 20 March 2017 on Information on Private Life and its Protection No. 519-V',
        'regions' => ['TM'],
        'continent' => 'Asia',
      ],
      'turkey' => [
        'name' => 'Turkey - Code on the Protection of Personal Data numbered 6698',
        'regions' => ['TR'],
        'continent' => 'Asia',
      ],
      'indonesia' => [
        'name' => 'Indonesia - Personal Data Protection Bill',
        'regions' => ['ID'],
        'continent' => 'Asia',
      ],
      'california' => [
        'name' => 'California - California Privacy Rights Act (CPRA)',
        'regions' => ['US'],
        'states' => ['CA'],
        'continent' => 'USA States',
      ],
      'virginia' => [
        'name' => 'Virginia - Consumer Data Protection Act (CDPA)',
        'regions' => ['US'],
        'states' => ['VA'],
        'continent' => 'USA States',
      ],
      'colorado' => [
        'name' => 'Colorado - Colorado Privacy Act',
        'regions' => ['US'],
        'states' => ['CO'],
        'continent' => 'USA States',
      ],
      'utah' => [
        'name' => 'Utah - Utah Consumer Privacy Act (UCPA)',
        'regions' => ['US'],
        'states' => ['UT'],
        'continent' => 'USA States',
      ],
      'oklahoma' => [
        'name' => 'Oklahoma - Uniform Personal Data Protection Act (UPDPA)',
        'regions' => ['US'],
        'states' => ['OK'],
        'continent' => 'USA States',
      ],
      'nebraska' => [
        'name' => 'Nebraska - Uniform Personal Data Protection Act (UPDPA)',
        'regions' => ['US'],
        'states' => ['NE'],
        'continent' => 'USA States',
      ],
      'nevada' => [
        'name' => 'Nevada - Nevada Privacy Law',
        'regions' => ['US'],
        'states' => ['NV'],
        'continent' => 'USA States',
      ],
      'district_of_columbia' => [
        'name' => 'District of Columbia - Uniform Personal Data Protection Act (UPDPA)',
        'regions' => ['US'],
        'states' => ['DC'],
        'continent' => 'USA States',
      ],
      'connecticut' => [
        'name' => 'Connecticut - The Connecticut Data Privacy Act (TCDPA)',
        'regions' => ['US'],
        'states' => ['CT'],
        'continent' => 'USA States',
      ],
      'canada' => [
        'name' => 'Canada - Personal Information Protection and Electronic Documents Act (PIPEDA)',
        'regions' => ['CA'],
        'continent' => 'North America',
      ],
      'british_virgin_islands' => [
        'name' => 'British Virgin Islands - Data Protection Act',
        'regions' => ['VG'],
        'continent' => 'North America',
      ],
      'argentina' => [
        'name' => 'Argentina - Personal Data Protection Act (PDPA)',
        'regions' => ['AR'],
        'continent' => 'South America',
      ],
      'peru' => [
        'name' => 'Peru - Law No. 29733 on the Protection of Personal Data',
        'regions' => ['PE'],
        'continent' => 'South America',
      ],
      'uruguay' => [
        'name' => 'Uruguay - Law No. 18.331 on the Protection of Personal Data',
        'regions' => ['UY'],
        'continent' => 'South America',
      ],
      'jamaica' => [
        'name' => 'Jamaica - Personal Data Protection Act (PDPA)',
        'regions' => ['JM'],
        'continent' => 'North America',
      ],
      'nicaragua' => [
        'name' => 'Nicaragua - Law on Personal Data Protection No. 787',
        'regions' => ['NI'],
        'continent' => 'North America',
      ],
      'brazil' => [
        'name' => 'Brazil - Lei Geral de Proteção de Dados Pessoais (LGPD)',
        'regions' => ['BR'],
        'continent' => 'South America',
      ],
      'mexico' => [
        'name' => 'Mexico - Federal Law on the Protection of Personal Data (LPPD)',
        'regions' => ['MX'],
        'continent' => 'North America',
      ],
      'barbados' => [
        'name' => 'Barbados - Barbados Data Protection Act (BDPA)',
        'regions' => ['BB'],
        'continent' => 'North America',
      ],
      'bermuda' => [
        'name' => 'Bermuda - Personal Information Protection Act (PIPA)',
        'regions' => ['BM'],
        'continent' => 'North America',
      ],
      'colombia' => [
        'name' => 'Colombia - Law Nº 1273 of 2009',
        'regions' => ['CO'],
        'continent' => 'South America',
      ],
      'costa_rica' => [
        'name' => 'Costa Rica - Law on the Protection of Persons Regarding the Processing of their Personal Data',
        'regions' => ['CR'],
        'continent' => 'North America',
      ],
      'cuba' => [
        'name' => 'Cuba - Personal Data Protection Law',
        'regions' => ['CU'],
        'continent' => 'North America',
      ],
      'curacao' => [
        'name' => 'Curaçao - Rules on the Protection of Personal Data',
        'regions' => ['CW'],
        'continent' => 'North America',
      ],
      'honduras' => [
        'name' => 'Honduras - Regulation of Transparency and Access to Public Information Law',
        'regions' => ['HN'],
        'continent' => 'North America',
      ],
      'cayman_islands' => [
        'name' => 'Cayman Islands - Data Protection Act',
        'regions' => ['KY'],
        'continent' => 'North America',
      ],
      'panama' => [
        'name' => 'Panama - Data Protection Law',
        'regions' => ['PA'],
        'continent' => 'North America',
      ],
      'trinidad_and_tobago' => [
        'name' => 'Trinidad and Tobago - Data Protection Act',
        'regions' => ['TT'],
        'continent' => 'North America',
      ],
      'south_korea' => [
        'name' => 'South Korea - Personal Information Protection Act (PIPA)',
        'regions' => ['KR'],
        'continent' => 'Asia',
      ],
      'china' => [
        'name' => 'China - Personal Information Protection Law (PIPL)',
        'regions' => ['CH'],
        'continent' => 'Asia',
      ],
      'hong_kong' => [
        'name' => 'Hong Kong - Personal Data Privacy Ordinance',
        'regions' => ['HK'],
        'continent' => 'Asia',
      ],
      'japan' => [
        'name' => 'Japan - Act on the Protection of Personal Information (LGPD)',
        'regions' => ['JP'],
        'continent' => 'Asia',
      ],
      'kazakhstan' => [
        'name' => 'Kazakhstan - Personal Data Law',
        'regions' => ['KZ'],
        'continent' => 'Asia',
      ],
      'bahrain' => [
        'name' => 'Bahrain - Personal Data Protection Law (PDPL)',
        'regions' => ['BH'],
        'continent' => 'Asia',
      ],
      'laos' => [
        'name' => 'Laos - Law on Electronic Data Protection',
        'regions' => ['LA'],
        'continent' => 'Asia',
      ],
      'oman' => [
        'name' => 'Oman - Personal Data Protection Law (PDPL)',
        'regions' => ['OM'],
        'continent' => 'Asia',
      ],
      'nigeria' => [
        'name' => 'Nigeria - Nigerian Data Protection Regulation (NDRP)',
        'regions' => ['NG'],
        'continent' => 'Africa',
      ],
      'madagascar' => [
        'name' => 'Madagascar - Data Protection Law',
        'regions' => ['MG'],
        'continent' => 'Africa',
      ],
      'mauritania' => [
        'name' => 'Mauritania - Law No. 2017 - 020 on the protection of the personal data',
        'regions' => ['MR'],
        'continent' => 'Africa',
      ],
      'morocco' => [
        'name' => 'Morocco - Protection of Individuals with Regard to the Processing of Personal Data',
        'regions' => ['MA'],
        'continent' => 'Africa',
      ],
      'benin' => [
        'name' => 'Benin - Protection of Personally Identifiable Information (PPII)',
        'regions' => ['BJ'],
        'continent' => 'Africa',
      ],
      'south_africa' => [
        'name' => 'South Africa - Protection of Personal Information Act (LGPD)',
        'regions' => ['ZA'],
        'continent' => 'Africa',
      ],
      'cape_verde' => [
        'name' => 'Cape Verde - Comissão Nacional de Proteção de Dados',
        'regions' => ['CV'],
        'continent' => 'Africa',
      ],
      'congo_republic' => [
        'name' => 'Congo Republic - Law on the Protection of Personal Data (LPPD)',
        'regions' => ['CG'],
        'continent' => 'Africa',
      ],
      'zambia' => [
        'name' => 'Zambia - Convention on Cyber Security and Personal Data Protection',
        'regions' => ['ZM'],
        'continent' => 'Africa',
      ],
      'rwanda' => [
        'name' => 'Rwanda - Law No. 058/2021 Relating to the Protection of Personal Data and Privacy',
        'regions' => ['RW'],
        'continent' => 'Africa',
      ],
      'zimbabwe' => [
        'name' => 'Zimbabwe - Data Protection Act No. 05/2021',
        'regions' => ['ZW'],
        'continent' => 'Africa',
      ],
      'burkina_faso' => [
        'name' => 'Burkina Faso - Data Protection Act',
        'regions' => ['BF'],
        'continent' => 'Africa',
      ],
      'kenya' => [
        'name' => 'Kenya - Data Protection Act',
        'regions' => ['KE'],
        'continent' => 'Africa',
      ],
      'chad' => [
        'name' => 'Chad - Data Protection Act',
        'regions' => ['TD'],
        'continent' => 'Africa',
      ],
      'niger' => [
        'name' => 'Niger - Data Protection Act',
        'regions' => ['NE'],
        'continent' => 'Africa',
      ],
      'algeria' => [
        'name' => 'Algeria - Law Relating to the Protection of Individuals in the Processing of Personal Data',
        'regions' => ['DZ'],
        'continent' => 'Africa',
      ],
      'egypt' => [
        'name' => 'Egypt - Data Protection Law',
        'regions' => ['EG'],
        'continent' => 'Africa',
      ],
      'ethiopia' => [
        'name' => 'Ethiopia - Data Protection Proclamation (draft)',
        'regions' => ['ET'],
        'continent' => 'Africa',
      ],
      'gabon' => [
        'name' => 'Gabon - Law on the Protection of Personal Data',
        'regions' => ['GA'],
        'continent' => 'Africa',
      ],
      'ghana' => [
        'name' => 'Ghana - Data Protection Act',
        'regions' => ['GH'],
        'continent' => 'Africa',
      ],
      'guinea' => [
        'name' => 'Guinea-Bissau - Law on Cybersecurity and Personal Data',
        'regions' => ['GW'],
        'continent' => 'Africa',
      ],
      'mali' => [
        'name' => 'Mali - Law No. 2013-015 of 21 May 2013 on the Protection of Personal Data',
        'regions' => ['ML'],
        'continent' => 'Africa',
      ],
      'mauritius' => [
        'name' => 'Mauritius - Data Protection Act 2017',
        'regions' => ['MU'],
        'continent' => 'Africa',
      ],
      'sao_tome_and_príncipe' => [
        'name' => 'São Tomé and Príncipe - Law No. 03/2016 on the Protection of Personal Data',
        'regions' => ['ST'],
        'continent' => 'Africa',
      ],
      'eswatini' => [
        'name' => 'Eswatini - Data Protection Act No. 41 of 2022',
        'regions' => ['SZ'],
        'continent' => 'Africa',
      ],
      'togo' => [
        'name' => 'Togo - Law No. 2019-014 Relating to the Protection of Personal Data',
        'regions' => ['TG'],
        'continent' => 'Africa',
      ],
      'uganda' => [
        'name' => 'Uganda - Data Protection and Privacy Act, 2019',
        'regions' => ['UG'],
        'continent' => 'Africa',
      ],
    ];

    $this->moduleHandler->alter('eu_cookie_compliance_regions', $cookie_compliance_regions);

    return $cookie_compliance_regions;
  }

  /**
   * Get sorted regions.
   *
   * @return array
   *   Sorted regions, grouped per continent.
   */
  public function getSortedRegions() : array {
    $raw_regions = $this->getRegions();
    ksort($raw_regions);

    $regions = [];

    foreach ($raw_regions as $key => $raw_region) {
      $regions[$raw_region['continent']][$key] = $raw_region;
    }

    ksort($regions);
    return $regions;

  }

  /**
   * Get the region codes for a set of preferences.
   *
   * @param string $cookie_compliance_set_id
   *   The stored countries/regions.
   *
   * @return array
   *   The ISO-3166 alpha2 codes.
   *
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   */
  public function getSelectedCountryCodes(string $cookie_compliance_set_id) : array {
    $cookie_compliance_set = $this->entityTypeManager->getStorage('cookie_compliance_set')->load($cookie_compliance_set_id);
    if ($cookie_compliance_set !== NULL) {

      $all_regions = $this->getRegions();
      $cc_set_regions = $cookie_compliance_set->get('regions');

      $cc_set_region_codes = [];
      foreach ($cc_set_regions as $region) {
        $cc_set_region_codes += $all_regions[$region]['regions'];
      }
      $custom_regions = $cookie_compliance_set->get('custom_regions');
      if ($custom_regions) {
        $custom_regions = explode(',', $custom_regions);
        foreach ($custom_regions as $custom_region) {
          if (strpos($custom_region, '/') !== FALSE) {
            $custom_region_code = explode('/', $custom_region);
            $cc_set_region_codes[] = reset($custom_region_code);
          }
          else {
            $cc_set_region_codes[] = $custom_region;
          }
        }
      }

      return $cc_set_region_codes;
    }
    return [];
  }

  /**
   * Get the sates codes for a set of preferences.
   *
   * @param string $cookie_compliance_set_id
   *   The stored countries/regions.
   *
   * @return array
   *   The ISO-3166 alpha2 codes.
   *
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   */
  public function getSelectedStateCodes(string $cookie_compliance_set_id) : array {
    $cookie_compliance_set = $this->entityTypeManager->getStorage('cookie_compliance_set')->load($cookie_compliance_set_id);
    if ($cookie_compliance_set !== NULL) {

      $all_regions = $this->getRegions();
      $cc_set_regions = $cookie_compliance_set->get('regions');

      $cc_set_region_codes = [];
      foreach ($cc_set_regions as $region) {
        if (!empty($all_regions[$region]['states'])) {
          $cc_set_region_codes += $all_regions[$region]['states'];
        }
      }
      $custom_regions = $cookie_compliance_set->get('custom_regions');
      if ($custom_regions) {
        $custom_regions = explode(',', $custom_regions);
        foreach ($custom_regions as $custom_region) {
          if (strpos($custom_region, '/') !== FALSE) {
            $custom_region_code = explode('/', $custom_region);
            $cc_set_region_codes[] = end($custom_region_code);
          }
          else {
            $cc_set_region_codes[] = $custom_region;
          }
        }
      }

      return $cc_set_region_codes;
    }
    return [];
  }

  /**
   * Find out if we are using regions.
   *
   * @return bool
   *   Whether we are using regions.
   *
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   */
  public function hasSetsWithRegions() :bool {
    $cookie_compliance_sets = $this->entityTypeManager->getStorage('cookie_compliance_set')->loadMultiple();
    foreach ($cookie_compliance_sets as $cookie_compliance_set) {
      if (!empty($cookie_compliance_set->get('regions')) || !empty($cookie_compliance_set->get('custom_regions'))) {
        return TRUE;
      }
    }
    return FALSE;
  }

}
